package de.joshavg

import javax.inject.Inject
import javax.ws.rs.*
import javax.ws.rs.core.MediaType
import javax.ws.rs.core.Response
import javax.ws.rs.core.Response.Status.PRECONDITION_REQUIRED

@Path("builds")
class Endpoints(
    @Inject
    val builds: Builds,
    @Inject
    val security: Security,
    @Inject
    val scriptRunner: ScriptRunner,
    @Inject
    val gitRunner: GitRunner,
    @Inject
    val handles: Handles
) {

    @GET
    @Path("{build}/info")
    @Produces(MediaType.APPLICATION_JSON)
    fun info(
        @PathParam("build")
        build: BuildId,
        @QueryParam("key")
        key: String?
    ) = security.requireKey(build, key) {
        it.asPublic()
    }

    @POST
    @Path("{build}/trigger")
    @Produces(MediaType.APPLICATION_JSON)
    fun trigger(
        @PathParam("build")
        build: BuildId,
        @QueryParam("key")
        key: String?,
        @QueryParam("rev")
        rev: String?
    ) = security.requireKey(build, key) {
        if (rev == null) {
            return@requireKey Response.status(PRECONDITION_REQUIRED).build()
        }

        val info = if (builds.buildConfig(build).script != null) {
            scriptRunner.run(build, rev)
        } else {
            gitRunner.run(build, rev)
        }

        Response.ok(
            mapOf(
                "log" to info.logFile.absolutePath,
                "pid" to info.pid
            )
        ).build()
    }

    @GET
    @Path("{build}/handles")
    @Produces(MediaType.APPLICATION_JSON)
    fun handles(
        @PathParam("build")
        build: BuildId,
        @QueryParam("key")
        key: String?
    ) = security.requireKey(build, key) {
        handles.active(build)
    }

}

