package de.joshavg

import org.slf4j.Logger
import org.slf4j.LoggerFactory
import java.io.File
import java.time.ZonedDateTime
import javax.enterprise.context.ApplicationScoped
import javax.inject.Inject

@ApplicationScoped
class ScriptRunner(
    @Inject
    val builds: Builds,
    @Inject
    val handles: Handles
) {

    fun run(buildId: BuildId, rev: String): ProcessInfo {
        val config = builds.buildConfig(buildId)
        val logFile = builds.createLogFile(buildId)

        logger.info("preparing process for build $buildId with config ${config.asPublic()}")
        logger.info("log file: $logFile")

        logFileHeader(logFile, buildId, rev)

        val process = ProcessBuilder(config.script)
            .directory(File(config.workspace))
            .redirectOutput(logFile)
            .redirectError(logFile)
            .apply {
                environment().putAll(config.env)
                environment()["ALFRED_LOG_FILE"] = logFile.absolutePath
                environment()["ALFRED_REV"] = rev
            }
            .start()
        handles.add(Handle(process.toHandle(), buildId))

        val pid = process.pid()

        logger.info("pid for build $buildId is: $pid")

        return ProcessInfo(pid = pid, logFile = logFile)
    }

    val logger: Logger = LoggerFactory.getLogger(this::class.java)

}

data class ProcessInfo(
    val pid: Long,
    val logFile: File
)

